/*

-- Reactor - Set Real Mass
--
-- Affects selected objects: sets their Reactor mass depending on their volume and specified density of their material.
--
-- written by Mayec Rancel
-- created: 2009-02-13
-- tested on 3ds max v.2009 (64b)


TODO:
- options to store density presets
- define minimum mass (to avoid Reactor instabilities)

*/

-- returns a list with 1.Volume (float) and 2.Center of Mass (point3) position of selected object
fn volumeAndCMass obj =
( 
	local Volume= 0.0 
	local Center= [0.0, 0.0, 0.0] 
	local theMesh = snapshotasmesh obj
	local numFaces = theMesh.numfaces 
	for i = 1 to numFaces do 
		( 
		local Face= getFace theMesh i 
		local vert2 = getVert theMesh Face.z 
		local vert1 = getVert theMesh Face.y 
		local vert0 = getVert theMesh Face.x 
		local dV = Dot (Cross (vert1 - vert0) (vert2 - vert0)) vert0
		Volume+= dV 
		Center+= (vert0 + vert1 + vert2) * dV 
	) 
	delete theMesh
	Volume /= 6 
	Center /= 24 
	Center /= Volume 
	#(Volume,Center) 
)

-- UPMeter is "units per meter"
fn setMassFromDensity theObj theDensity UPMeter =
(
	theVolume = (volumeAndCMass theObj)[1] / (UPMeter^3)
	theMass = theVolume * theDensity
	setUserProp theObj "Mass" theMass
)

-- Initialize array that stores selection objects' names
selectArray = #()


rollout Reactor_setRealMass "Reactor: Set Real Mass" width:222 height:404
(
	
	spinner spn_upm "Units / Meter" pos:[20,22] width:180 height:16 type:#float range:[0.001,10000,reactor.worldScale]
	spinner spn_density "Mat. Density (Kg/m3)" pos:[48,57] width:155 height:16 type:#float range:[0.1,20000,7850]
	button btn_PropEditor "Rigid Body Properties Editor" pos:[30,88] width:160 height:25
	on btn_PropEditor pressed do
		macros.run "reactor" "rctOpenPropertyEditor"
	
	listBox lbx_selected "Selected Objects" pos:[20,125] width:181 height:11 items:selectArray
	-- updates the selected objects list
	fn updateSelectionList = (
		selectArray = #()
		for obj in selection do append selectArray obj.name
		lbx_selected.items = selectArray
	)
	
	button btn_setMasses "Set Reactor Masses" pos:[45,318] width:130 height:25
	on btn_setMasses pressed do
	(
		for o in selection do
		(
			print spn_density.value
			print spn_upm.value
			setMassFromDensity o spn_density.value spn_upm.value
		)
	)
	
	button btn_about "About" pos:[59,358] width:103 height:27
	on btn_about pressed do
		messagebox "Affects selected objects: sets their Reactor mass depending on their volume and specified density of their material.\nv1.0\nWritten by Mayec Rancel\nhttp://www.mayecrancel.com"

	-- when dialog is closed, remove callback that updates the "selected objects" list
	on Reactor_setRealMass close do callbacks.removeScripts #selectionSetChanged id:#setRealMass 
)

-- Launch MAIN DIALOG
createDialog Reactor_setRealMass
-- initialize selected objects list, and its callback to update on selection change
Reactor_setRealMass.updateSelectionList()
callbacks.addScript #selectionSetChanged "Reactor_setRealMass.updateSelectionList()" id:#setRealMass